import os
import time
import uuid

import pytest
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import ResourceNotFoundError
from azure.search.documents.indexes import SearchIndexClient
from haystack.document_stores.types import DuplicatePolicy

from haystack_integrations.document_stores.azure_ai_search import AzureAISearchDocumentStore

# This is the approximate time in seconds it takes for the documents to be available in Azure Search index
SLEEP_TIME_IN_SECONDS = 10
MAX_WAIT_TIME_FOR_INDEX_DELETION = 10


@pytest.fixture()
def sleep_time():
    return SLEEP_TIME_IN_SECONDS


@pytest.fixture
def document_store(request):
    """
    This is the most basic requirement for the child class: provide
    an instance of this document store so the base class can use it.
    """
    index_name = f"haystack_test_{uuid.uuid4().hex}"
    metadata_fields = getattr(request, "param", {}).get("metadata_fields", None)
    include_search_metadata = getattr(request, "param", {}).get("include_search_metadata", False)

    azure_endpoint = os.environ["AZURE_AI_SEARCH_ENDPOINT"]
    api_key = os.environ["AZURE_AI_SEARCH_API_KEY"]

    client = SearchIndexClient(azure_endpoint, AzureKeyCredential(api_key))
    if index_name in client.list_index_names():
        client.delete_index(index_name)

    store = AzureAISearchDocumentStore(
        index_name=index_name,
        create_index=True,
        embedding_dimension=768,
        metadata_fields=metadata_fields,
        include_search_metadata=include_search_metadata,
    )

    # Override some methods to wait for the documents to be available
    original_write_documents = store.write_documents
    original_delete_documents = store.delete_documents
    original_delete_all_documents = store.delete_all_documents
    original_delete_by_filter = store.delete_by_filter
    original_update_by_filter = store.update_by_filter

    def write_documents_and_wait(documents, policy=DuplicatePolicy.OVERWRITE):
        written_docs = original_write_documents(documents, policy)
        time.sleep(SLEEP_TIME_IN_SECONDS)
        return written_docs

    def delete_documents_and_wait(filters):
        original_delete_documents(filters)
        time.sleep(SLEEP_TIME_IN_SECONDS)

    def delete_all_documents_and_wait():
        original_delete_all_documents()
        time.sleep(SLEEP_TIME_IN_SECONDS)

    def delete_by_filter_and_wait(filters):
        deleted_count = original_delete_by_filter(filters)
        time.sleep(SLEEP_TIME_IN_SECONDS)
        return deleted_count

    def update_by_filter_and_wait(filters, fields):
        updated_count = original_update_by_filter(filters, fields)
        time.sleep(SLEEP_TIME_IN_SECONDS)
        return updated_count

    # Helper function to wait for the index to be deleted, needed to cover latency
    def wait_for_index_deletion(client, index_name):
        start_time = time.time()
        while time.time() - start_time < MAX_WAIT_TIME_FOR_INDEX_DELETION:
            if index_name not in client.list_index_names():
                return True
            time.sleep(1)
        return False

    store.write_documents = write_documents_and_wait
    store.delete_documents = delete_documents_and_wait
    store.delete_all_documents = delete_all_documents_and_wait
    store.delete_by_filter = delete_by_filter_and_wait
    store.update_by_filter = update_by_filter_and_wait

    yield store
    try:
        client.delete_index(index_name)
        if not wait_for_index_deletion(client, index_name):
            print(f"Index {index_name} was not properly deleted.")
    except ResourceNotFoundError:
        print(f"Index {index_name} was already deleted or not found.")
    except Exception as e:
        print(f"Unexpected error when deleting index {index_name}: {e}")
        raise


@pytest.fixture(scope="session", autouse=True)
def cleanup_indexes():
    """
    Fixture to clean up all remaining indexes at the end of the test session.
    Only runs if Azure credentials are available.
    Automatically runs after all tests.
    """
    azure_endpoint = os.getenv("AZURE_AI_SEARCH_ENDPOINT")
    api_key = os.getenv("AZURE_AI_SEARCH_API_KEY")

    # Skip cleanup if credentials aren't available
    if not azure_endpoint or not api_key:
        yield
        return

    client = SearchIndexClient(azure_endpoint, AzureKeyCredential(api_key))

    yield  # Allow tests to run before performing cleanup

    # Cleanup: Delete all remaining indexes
    print("Starting session-level cleanup of all Azure Search indexes.")
    existing_indexes = client.list_index_names()
    for index in existing_indexes:
        try:
            client.delete_index(index)
        except Exception as e:
            print(f"Failed to delete index during clean up {index}: {e}")



from haystack import Document

from haystack_integrations.document_stores.azure_ai_search import AzureAISearchDocumentStore

"""
This example demonstrates how to use the AzureAISearchDocumentStore to write and filter documents.
To run this example, you'll need an Azure Search service endpoint and API key, which can either be
set as environment variables (AZURE_AI_SEARCH_ENDPOINT and AZURE_AI_SEARCH_API_KEY) or
provided directly to AzureAISearchDocumentStore(as params "api_key", "azure_endpoint").
Otherwise you can use DefaultAzureCredential to authenticate with Azure services.
See more details at https://learn.microsoft.com/en-us/azure/search/keyless-connections?tabs=python%2Cazure-cli
"""
document_store = AzureAISearchDocumentStore(
    metadata_fields={"version": float, "label": str},
    index_name="document-store-example",
)

documents = [
    Document(
        content="This is an introduction to using Python for data analysis.",
        meta={"version": 1.0, "label": "chapter_one"},
    ),
    Document(
        content="Learn how to use Python libraries for machine learning.",
        meta={"version": 1.5, "label": "chapter_two"},
    ),
    Document(
        content="Advanced Python techniques for data visualization.",
        meta={"version": 2.0, "label": "chapter_three"},
    ),
]
document_store.write_documents(documents)

filters = {
    "operator": "AND",
    "conditions": [
        {"field": "meta.version", "operator": ">", "value": 1.2},
        {"field": "meta.label", "operator": "in", "value": ["chapter_one", "chapter_three"]},
    ],
}

results = document_store.filter_documents(filters)
print(results)



from .bm25_retriever import AzureAISearchBM25Retriever
from .embedding_retriever import AzureAISearchEmbeddingRetriever
from .hybrid_retriever import AzureAISearchHybridRetriever

__all__ = ["AzureAISearchBM25Retriever", "AzureAISearchEmbeddingRetriever", "AzureAISearchHybridRetriever"]



from haystack.document_stores.errors import DocumentStoreError
from haystack.errors import FilterError


class AzureAISearchDocumentStoreError(DocumentStoreError):
    """Parent class for all AzureAISearchDocumentStore exceptions."""

    pass


class AzureAISearchDocumentStoreConfigError(AzureAISearchDocumentStoreError):
    """Raised when a configuration is not valid for a AzureAISearchDocumentStore."""

    pass


class AzureAISearchDocumentStoreFilterError(FilterError):
    """Raised when filter is not valid for AzureAISearchDocumentStore."""

    pass



# SPDX-FileCopyrightText: 2023-present deepset GmbH <info@deepset.ai>
#
# SPDX-License-Identifier: Apache-2.0

import logging as python_logging
from datetime import datetime
from typing import Any

from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceNotFoundError,
)
from azure.core.pipeline.policies import UserAgentPolicy
from azure.identity import DefaultAzureCredential
from azure.search.documents import SearchClient
from azure.search.documents.indexes import SearchIndexClient
from azure.search.documents.indexes.models import (
    CharFilter,
    CorsOptions,
    HnswAlgorithmConfiguration,
    HnswParameters,
    LexicalAnalyzer,
    LexicalTokenizer,
    ScoringProfile,
    SearchableField,
    SearchField,
    SearchFieldDataType,
    SearchIndex,
    SearchResourceEncryptionKey,
    SearchSuggester,
    SimilarityAlgorithm,
    SimpleField,
    TokenFilter,
    VectorSearch,
    VectorSearchAlgorithmMetric,
    VectorSearchProfile,
)
from azure.search.documents.models import VectorizedQuery
from haystack import default_from_dict, default_to_dict, logging
from haystack.dataclasses import Document
from haystack.document_stores.types import DuplicatePolicy
from haystack.utils import Secret, deserialize_secrets_inplace

from .errors import AzureAISearchDocumentStoreConfigError, AzureAISearchDocumentStoreError
from .filters import _normalize_filters

USER_AGENT = "haystack-integrations/azure-ai-search"
BIG_TOP_K = 100000

type_mapping = {
    str: "Edm.String",
    bool: "Edm.Boolean",
    int: "Edm.Int32",
    float: "Edm.Double",
    datetime: "Edm.DateTimeOffset",
}

# Map of expected field names to their corresponding classes
AZURE_CLASS_MAPPING: dict[str, Any] = {
    "suggesters": SearchSuggester,
    "analyzers": LexicalAnalyzer,
    "tokenizers": LexicalTokenizer,
    "token_filters": TokenFilter,
    "char_filters": CharFilter,
    "cors_options": CorsOptions,
    "similarity_algorithm": SimilarityAlgorithm,
    "encryption_key": SearchResourceEncryptionKey,
    "scoring_profiles": ScoringProfile,
}

DEFAULT_VECTOR_SEARCH = VectorSearch(
    profiles=[
        VectorSearchProfile(
            name="default-vector-config",
            algorithm_configuration_name="cosine-algorithm-config",
        )
    ],
    algorithms=[
        HnswAlgorithmConfiguration(
            name="cosine-algorithm-config",
            parameters=HnswParameters(
                metric=VectorSearchAlgorithmMetric.COSINE,
            ),
        )
    ],
)

logger = logging.getLogger(__name__)
python_logging.getLogger("azure").setLevel(python_logging.ERROR)
python_logging.getLogger("azure.identity").setLevel(python_logging.DEBUG)


class AzureAISearchDocumentStore:
    def __init__(
        self,
        *,
        api_key: Secret = Secret.from_env_var("AZURE_AI_SEARCH_API_KEY", strict=False),  # noqa: B008
        azure_endpoint: Secret = Secret.from_env_var("AZURE_AI_SEARCH_ENDPOINT", strict=True),  # noqa: B008
        index_name: str = "default",
        embedding_dimension: int = 768,
        metadata_fields: dict[str, SearchField | type] | None = None,
        vector_search_configuration: VectorSearch | None = None,
        include_search_metadata: bool = False,
        **index_creation_kwargs: Any,
    ):
        """
        A document store using [Azure AI Search](https://azure.microsoft.com/products/ai-services/ai-search/)
        as the backend.

        :param azure_endpoint: The URL endpoint of an Azure AI Search service.
        :param api_key: The API key to use for authentication.
        :param index_name: Name of index in Azure AI Search, if it doesn't exist it will be created.
        :param embedding_dimension: Dimension of the embeddings.
        :param metadata_fields: A dictionary mapping metadata field names to their corresponding field definitions.
            Each field can be defined either as:
            - A SearchField object to specify detailed field configuration like type, searchability, and filterability
            - A Python type (`str`, `bool`, `int`, `float`, or `datetime`) to create a simple filterable field

            These fields are automatically added when creating the search index.
            Example:
            ```python
            metadata_fields={
                "Title": SearchField(
                    name="Title",
                    type="Edm.String",
                    searchable=True,
                    filterable=True
                ),
                "Pages": int
            }
            ```
        :param vector_search_configuration: Configuration option related to vector search.
            Default configuration uses the HNSW algorithm with cosine similarity to handle vector searches.

        :param include_search_metadata: Whether to include Azure AI Search metadata fields
            in the returned documents. When set to True, the `meta` field of the returned
            documents will contain the @search.score, @search.reranker_score, @search.highlights,
            @search.captions, and other fields returned by Azure AI Search.
        :param index_creation_kwargs: Optional keyword parameters to be passed to `SearchIndex` class
            during index creation. Some of the supported parameters:
                - `semantic_search`: Defines semantic configuration of the search index. This parameter is needed
                to enable semantic search capabilities in index.
                - `similarity`: The type of similarity algorithm to be used when scoring and ranking the documents
                matching a search query. The similarity algorithm can only be defined at index creation time and
                cannot be modified on existing indexes.

        For more information on parameters, see the [official Azure AI Search documentation](https://learn.microsoft.com/en-us/azure/search/).
        """
        self._client: SearchClient | None = None
        self._index_client: SearchIndexClient | None = None
        self._index_fields = []  # type: list[Any]  # stores all fields in the final schema of index
        self._api_key = api_key
        self._azure_endpoint = azure_endpoint
        self._index_name = index_name
        self._embedding_dimension = embedding_dimension
        self._dummy_vector = [-10.0] * self._embedding_dimension
        self._metadata_fields = AzureAISearchDocumentStore._normalize_metadata_index_fields(metadata_fields)
        self._vector_search_configuration = vector_search_configuration or DEFAULT_VECTOR_SEARCH
        self._include_search_metadata = include_search_metadata
        self._index_creation_kwargs = index_creation_kwargs

    @property
    def client(self) -> SearchClient:
        resolved_endpoint = self._azure_endpoint.resolve_value()
        resolved_key = self._api_key.resolve_value()

        credential = AzureKeyCredential(resolved_key) if resolved_key else DefaultAzureCredential()

        # build a UserAgentPolicy to be used for the request
        ua_policy = UserAgentPolicy(user_agent=USER_AGENT)
        try:
            if not self._index_client:
                self._index_client = SearchIndexClient(
                    # resolve_value, with Secret.from_env_var (strict=True), returns a string or raises an error
                    endpoint=resolved_endpoint,  # type: ignore[arg-type]
                    credential=credential,
                    user_agent=ua_policy,
                )
            if not self._index_exists(self._index_name):
                # Create a new index if it does not exist
                logger.debug(
                    "The index '{idx_name}' does not exist. A new index will be created.",
                    idx_name=self._index_name,
                )
                self._create_index()
        except (HttpResponseError, ClientAuthenticationError) as error:
            msg = f"Failed to authenticate with Azure Search: {error}"
            raise AzureAISearchDocumentStoreConfigError(msg) from error

        if self._index_client:
            # Get the search client, if index client is initialized
            index_fields = self._index_client.get_index(self._index_name).fields
            self._index_fields = [field.name for field in index_fields]
            self._client = self._index_client.get_search_client(self._index_name, user_agent=ua_policy)
        else:
            msg = "Search Index Client is not initialized."
            raise AzureAISearchDocumentStoreConfigError(msg)

        return self._client

    @staticmethod
    def _normalize_metadata_index_fields(
        metadata_fields: dict[str, SearchField | type] | None,
    ) -> dict[str, SearchField]:
        """Create a list of index fields for storing metadata values."""

        if not metadata_fields:
            return {}

        normalized_fields = {}

        for key, value in metadata_fields.items():
            if isinstance(value, SearchField):
                if value.name == key:
                    normalized_fields[key] = value
                else:
                    msg = f"Name of SearchField ('{value.name}') must match metadata field name ('{key}')"
                    raise ValueError(msg)
            else:
                if not key[0].isalpha():
                    msg = (
                        f"Azure Search index only allows field names starting with letters. "
                        f"Invalid key: {key} will be dropped."
                    )
                    logger.warning(msg)
                    continue

                field_type = type_mapping.get(value)
                if not field_type:
                    error_message = f"Unsupported field type for key '{key}': {value}"
                    raise ValueError(error_message)

                normalized_fields[key] = SimpleField(
                    name=key,
                    type=field_type,
                    filterable=True,
                )

        return normalized_fields

    def _create_index(self) -> None:
        """
        Internally creates a new search index.
        """

        # default fields to create index based on Haystack Document (id, content, embedding)
        default_fields = [
            SimpleField(name="id", type=SearchFieldDataType.String, key=True, filterable=True),
            SearchableField(name="content", type=SearchFieldDataType.String),
            SearchField(
                name="embedding",
                type=SearchFieldDataType.Collection(SearchFieldDataType.Single),
                searchable=True,
                hidden=False,
                vector_search_dimensions=self._embedding_dimension,
                vector_search_profile_name="default-vector-config",
            ),
        ]

        if self._metadata_fields:
            default_fields.extend(self._metadata_fields.values())

        index = SearchIndex(
            name=self._index_name,
            fields=default_fields,
            vector_search=self._vector_search_configuration,
            **self._index_creation_kwargs,
        )

        if self._index_client:
            self._index_client.create_index(index)

    @staticmethod
    def _serialize_index_creation_kwargs(
        index_creation_kwargs: dict[str, Any],
    ) -> dict[str, Any]:
        """
        Serializes the index creation kwargs to a dictionary.
        This is needed to handle serialization of Azure AI Search classes
        that are passed in the index creation kwargs.
        """
        result = {}
        for key, value in index_creation_kwargs.items():
            if hasattr(value, "as_dict"):
                result[key] = value.as_dict()
            elif isinstance(value, list) and all(hasattr(item, "as_dict") for item in value):
                result[key] = [item.as_dict() for item in value]
            else:
                result[key] = value
        return result

    def to_dict(self) -> dict[str, Any]:
        """
        Serializes the component to a dictionary.

        :returns:
            Dictionary with serialized data.
        """
        return default_to_dict(
            self,
            azure_endpoint=(self._azure_endpoint.to_dict() if self._azure_endpoint else None),
            api_key=self._api_key.to_dict() if self._api_key else None,
            index_name=self._index_name,
            embedding_dimension=self._embedding_dimension,
            metadata_fields={key: value.as_dict() for key, value in self._metadata_fields.items()},
            vector_search_configuration=self._vector_search_configuration.as_dict(),
            **self._serialize_index_creation_kwargs(self._index_creation_kwargs),
        )

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "AzureAISearchDocumentStore":
        """
        Deserializes the component from a dictionary.

        :param data:
            Dictionary to deserialize from.

        :returns:
            Deserialized component.
        """
        if (fields := data["init_parameters"]["metadata_fields"]) is not None:
            data["init_parameters"]["metadata_fields"] = {
                key: SearchField.from_dict(field) for key, field in fields.items()
            }
        else:
            data["init_parameters"]["metadata_fields"] = {}

        for key, model_class in AZURE_CLASS_MAPPING.items():
            if key in data["init_parameters"]:
                value = data["init_parameters"][key]
                if isinstance(value, list):
                    data["init_parameters"][key] = [model_class.from_dict(item) for item in value]
                else:
                    data["init_parameters"][key] = model_class.from_dict(value)

        deserialize_secrets_inplace(data["init_parameters"], keys=["api_key", "azure_endpoint"])
        if (vector_search_configuration := data["init_parameters"].get("vector_search_configuration")) is not None:
            data["init_parameters"]["vector_search_configuration"] = VectorSearch.from_dict(vector_search_configuration)
        return default_from_dict(cls, data)

    def count_documents(self) -> int:
        """
        Returns how many documents are present in the search index.

        :returns: list of retrieved documents.
        """
        return self.client.get_document_count()

    def write_documents(self, documents: list[Document], policy: DuplicatePolicy = DuplicatePolicy.NONE) -> int:
        """
        Writes the provided documents to search index.

        :param documents: documents to write to the index.
        :param policy: Policy to determine how duplicates are handled.
        :raises ValueError: If the documents are not of type Document.
        :raises TypeError: If the document ids are not strings.
        :return: the number of documents added to index.
        """

        if len(documents) > 0:
            if not isinstance(documents[0], Document):
                msg = "param 'documents' must contain a list of objects of type Document"
                raise ValueError(msg)

        if policy not in [DuplicatePolicy.NONE, DuplicatePolicy.OVERWRITE]:
            logger.warning(
                f"AzureAISearchDocumentStore only supports `DuplicatePolicy.OVERWRITE`"
                f"but got {policy}. Overwriting duplicates is enabled by default."
            )
        client = self.client
        documents_to_write = [self._convert_haystack_document_to_azure(doc) for doc in documents]

        if documents_to_write != []:
            client.upload_documents(documents_to_write)
        return len(documents_to_write)

    def delete_documents(self, document_ids: list[str]) -> None:
        """
        Deletes all documents with a matching document_ids from the search index.

        :param document_ids: ids of the documents to be deleted.
        """
        if self.count_documents() == 0:
            return
        documents = self._get_raw_documents_by_id(document_ids)
        if documents:
            self.client.delete_documents(documents)

    def delete_all_documents(self, recreate_index: bool = False) -> None:  # noqa: FBT002, FBT001
        """
        Deletes all documents in the document store.

        :param recreate_index: If True, the index will be deleted and recreated with the original schema.
            If False, all documents will be deleted while preserving the index.
        """
        try:
            if recreate_index:
                # Get current index definition
                if self._index_client is None:
                    msg = "Index client is not initialized"
                    raise ValueError(msg)
                current_index = self._index_client.get_index(self._index_name)

                # Delete and recreate index
                self._index_client.delete_index(self._index_name)
                self._index_client.create_index(current_index)
                logger.info("Index '{idx_name}' recreated with original schema.", idx_name=self._index_name)
            else:
                # Delete all documents without recreating index
                if self.count_documents() == 0:
                    return

                # Search for all documents (pagination handled by Azure SDK)
                all_docs = list(self.client.search(search_text="*", select=["id"], top=BIG_TOP_K))

                if all_docs:
                    self.client.delete_documents(all_docs)
                    logger.info(
                        "Deleted {n_docs} documents from index '{idx_name}'.",
                        n_docs=len(all_docs),
                        idx_name=self._index_name,
                    )
        except Exception as e:
            msg = f"Failed to delete all documents from Azure AI Search: {e!s}"
            raise HttpResponseError(msg) from e

    def delete_by_filter(self, filters: dict[str, Any]) -> int:
        """
        Deletes all documents that match the provided filters.

        Azure AI Search does not support server-side delete by query, so this method
        first searches for matching documents, then deletes them in a batch operation.

        :param filters: The filters to apply to select documents for deletion.
            For filter syntax, see [Haystack metadata filtering](https://docs.haystack.deepset.ai/docs/metadata-filtering)
        :returns: The number of documents deleted.
        """
        try:
            normalized_filters = _normalize_filters(filters)

            results = list(self.client.search(search_text="*", filter=normalized_filters, select=["id"], top=BIG_TOP_K))

            if not results:
                return 0

            documents_to_delete = [{"id": doc["id"]} for doc in results]
            self.client.delete_documents(documents=documents_to_delete)

            return len(documents_to_delete)

        except Exception as e:
            msg = f"Failed to delete documents by filter from Azure AI Search: {e!s}"
            raise AzureAISearchDocumentStoreError(msg) from e

    def update_by_filter(self, filters: dict[str, Any], meta: dict[str, Any]) -> int:
        """
        Updates the fields of all documents that match the provided filters.

        Azure AI Search does not support server-side update by query, so this method
        first searches for matching documents, then updates them using merge operations.

        :param filters: The filters to apply to select documents for updating.
            For filter syntax, see [Haystack metadata filtering](https://docs.haystack.deepset.ai/docs/metadata-filtering)
        :param meta: The fields to update. These fields must exist in the index schema.
        :returns: The number of documents updated.
        """
        try:
            # validate that fields to update exist in the index schema
            invalid_fields = [key for key in meta.keys() if key not in self._index_fields]
            if invalid_fields:
                msg = f"Fields {invalid_fields} are not defined in index schema. Available fields: {self._index_fields}"
                raise ValueError(msg)

            normalized_filters = _normalize_filters(filters)

            results = list(self.client.search(search_text="*", filter=normalized_filters, select=["id"], top=BIG_TOP_K))

            if not results:
                return 0

            documents_to_update = []
            for doc in results:
                update_doc = {"id": doc["id"]}
                update_doc.update(meta)
                documents_to_update.append(update_doc)

            self.client.merge_documents(documents=documents_to_update)

            logger.info(
                "Updated {n_docs} documents in index '{idx_name}' using filters.",
                n_docs=len(documents_to_update),
                idx_name=self._index_name,
            )
            return len(documents_to_update)

        except ValueError:
            # Re-raise ValueError for invalid fields without wrapping
            raise
        except Exception as e:
            msg = f"Failed to delete documents by filter from Azure AI Search: {e!s}"
            raise AzureAISearchDocumentStoreError(msg) from e

    def get_documents_by_id(self, document_ids: list[str]) -> list[Document]:
        return self._convert_search_result_to_documents(self._get_raw_documents_by_id(document_ids))

    def search_documents(self, search_text: str = "*", top_k: int = 10) -> list[Document]:
        """
        Returns all documents that match the provided search_text.
        If search_text is None, returns all documents.
        :param search_text: the text to search for in the Document list.
        :param top_k: Maximum number of documents to return.
        :returns: A list of Documents that match the given search_text.
        """
        result = self.client.search(search_text=search_text, top=top_k)
        return self._convert_search_result_to_documents(list(result))

    def filter_documents(self, filters: dict[str, Any] | None = None) -> list[Document]:
        """
        Returns the documents that match the provided filters.
        Filters should be given as a dictionary supporting filtering by metadata. For details on
        filters, see the [metadata filtering documentation](https://docs.haystack.deepset.ai/docs/metadata-filtering).

        :param filters: the filters to apply to the document list.
        :returns: A list of Documents that match the given filters.
        """
        if filters:
            normalized_filters = _normalize_filters(filters)
            result = self.client.search(filter=normalized_filters)
            return self._convert_search_result_to_documents(list(result))
        else:
            return self.search_documents()

    def _convert_search_result_to_documents(self, azure_docs: list[dict[str, Any]]) -> list[Document]:
        """
        Converts Azure search results to Haystack Documents.
        """
        documents = []

        for azure_doc in azure_docs:
            embedding = azure_doc.get("embedding")
            score = azure_doc.get("@search.score", None)
            if embedding == self._dummy_vector:
                embedding = None
            meta = {}

            # Anything besides default fields (id, content, and embedding) is considered metadata
            if self._include_search_metadata:
                meta = {key: value for key, value in azure_doc.items() if key not in ["id", "content", "embedding"]}
            else:
                meta = {
                    key: value
                    for key, value in azure_doc.items()
                    if key not in ["id", "content", "embedding"] and key in self._index_fields and value is not None
                }

            # Create the document with meta only if it's non-empty
            doc = Document(
                id=azure_doc["id"],
                content=azure_doc["content"],
                embedding=embedding,
                meta=meta,
                score=score,
            )

            documents.append(doc)
        return documents

    def _index_exists(self, index_name: str | None) -> bool:
        """
        Check if the index exists in the Azure AI Search service.

        :param index_name: The name of the index to check.
        :returns bool: whether the index exists.
        """

        if self._index_client and index_name:
            return index_name in self._index_client.list_index_names()
        else:
            msg = "Index name is required to check if the index exists."
            raise ValueError(msg)

    def _get_raw_documents_by_id(self, document_ids: list[str]) -> list[dict]:
        """
        Retrieves all Azure documents with a matching document_ids from the document store.

        :param document_ids: ids of the documents to be retrieved.
        :returns: list of retrieved Azure documents.
        """
        azure_documents = []
        for doc_id in document_ids:
            try:
                document = self.client.get_document(doc_id)
                azure_documents.append(document)
            except ResourceNotFoundError:
                logger.warning(f"Document with ID {doc_id} not found.")
        return azure_documents

    def _convert_haystack_document_to_azure(self, document: Document) -> dict[str, Any]:
        """Convert a Haystack Document to an Azure Search document"""

        doc_dict = document.to_dict(flatten=False)

        # Because Azure Search does not allow dynamic fields, we only include fields that are part of the schema
        index_document = {k: v for k, v in {**doc_dict, **doc_dict.get("meta", {})}.items() if k in self._index_fields}
        if index_document["embedding"] is None:
            index_document["embedding"] = self._dummy_vector

        return index_document

    def _embedding_retrieval(
        self,
        query_embedding: list[float],
        *,
        top_k: int = 10,
        filters: str | None = None,
        **kwargs: Any,
    ) -> list[Document]:
        """
        Retrieves documents that are most similar to the query embedding using a vector similarity metric.
        It uses the vector configuration specified in the document store. By default, it uses the HNSW algorithm
        with cosine similarity.

        This method is not meant to be part of the public interface of
        `AzureAISearchDocumentStore` nor called directly.
        `AzureAISearchEmbeddingRetriever` uses this method directly and is the public interface for it.

        :param query_embedding: Embedding of the query.
        :param top_k: Maximum number of Documents to return.
        :param filters: Filters applied to the retrieved Documents.
        :param kwargs: Optional keyword arguments to pass to the Azure AI's search endpoint.

        :raises ValueError: If `query_embedding` is an empty list.
        :returns: List of Document that are most similar to `query_embedding`.
        """

        if not query_embedding:
            msg = "query_embedding must be a non-empty list of floats"
            raise ValueError(msg)

        vector_query = VectorizedQuery(vector=query_embedding, k_nearest_neighbors=top_k, fields="embedding")
        result = self.client.search(vector_queries=[vector_query], filter=filters, **kwargs)
        azure_docs = list(result)
        return self._convert_search_result_to_documents(azure_docs)

    def _bm25_retrieval(
        self,
        query: str,
        top_k: int = 10,
        filters: str | None = None,
        **kwargs: Any,
    ) -> list[Document]:
        """
        Retrieves documents that are most similar to `query`, using the BM25 algorithm.

        This method is not meant to be part of the public interface of
        `AzureAISearchDocumentStore` nor called directly.
        `AzureAISearchBM25Retriever` uses this method directly and is the public interface for it.

        :param query: Text of the query.
        :param filters: Filters applied to the retrieved Documents.
        :param top_k: Maximum number of Documents to return.
        :param kwargs: Optional keyword arguments to pass to the Azure AI's search endpoint.


        :raises ValueError: If `query` is an empty string.
        :returns: List of Document that are most similar to `query`.
        """

        if query is None:
            msg = "query must not be None"
            raise ValueError(msg)

        result = self.client.search(search_text=query, filter=filters, top=top_k, **kwargs)
        azure_docs = list(result)
        return self._convert_search_result_to_documents(azure_docs)

    def _hybrid_retrieval(
        self,
        query: str,
        query_embedding: list[float],
        top_k: int = 10,
        filters: str | None = None,
        **kwargs: Any,
    ) -> list[Document]:
        """
        Retrieves documents similar to query using the vector configuration in the document store and
        the BM25 algorithm. This method combines vector similarity and BM25 for improved retrieval.

        This method is not meant to be part of the public interface of
        `AzureAISearchDocumentStore` nor called directly.
        `AzureAISearchHybridRetriever` uses this method directly and is the public interface for it.

        :param query: Text of the query.
        :param query_embedding: Embedding of the query.
        :param filters: Filters applied to the retrieved Documents.
        :param top_k: Maximum number of Documents to return.
        :param kwargs: Optional keyword arguments to pass to the Azure AI's search endpoint.

        :raises ValueError: If `query` or `query_embedding` is empty.
        :returns: List of Document that are most similar to `query`.
        """

        if query is None:
            msg = "query must not be None"
            raise ValueError(msg)
        if not query_embedding:
            msg = "query_embedding must be a non-empty list of floats"
            raise ValueError(msg)

        vector_query = VectorizedQuery(vector=query_embedding, k_nearest_neighbors=top_k, fields="embedding")
        result = self.client.search(
            search_text=query,
            vector_queries=[vector_query],
            filter=filters,
            top=top_k,
            **kwargs,
        )
        azure_docs = list(result)
        return self._convert_search_result_to_documents(azure_docs)



# SPDX-FileCopyrightText: 2023-present deepset GmbH <info@deepset.ai>
#
# SPDX-License-Identifier: Apache-2.0
import os
from unittest.mock import Mock

import pytest
from azure.core.exceptions import HttpResponseError
from haystack.dataclasses import Document
from haystack.document_stores.types import FilterPolicy
from numpy.random import rand  # type: ignore

from haystack_integrations.components.retrievers.azure_ai_search import AzureAISearchHybridRetriever
from haystack_integrations.document_stores.azure_ai_search import DEFAULT_VECTOR_SEARCH, AzureAISearchDocumentStore


def test_init_default():
    mock_store = Mock(spec=AzureAISearchDocumentStore)
    retriever = AzureAISearchHybridRetriever(document_store=mock_store)
    assert retriever._document_store == mock_store
    assert retriever._filters == {}
    assert retriever._top_k == 10
    assert retriever._filter_policy == FilterPolicy.REPLACE

    retriever = AzureAISearchHybridRetriever(document_store=mock_store, filter_policy="replace")
    assert retriever._filter_policy == FilterPolicy.REPLACE

    with pytest.raises(ValueError):
        AzureAISearchHybridRetriever(document_store=mock_store, filter_policy="unknown")


def test_to_dict():
    document_store = AzureAISearchDocumentStore(hosts="some fake host")
    retriever = AzureAISearchHybridRetriever(document_store=document_store)
    res = retriever.to_dict()
    assert res == {
        "type": "haystack_integrations.components.retrievers.azure_ai_search.hybrid_retriever.AzureAISearchHybridRetriever",  # noqa: E501
        "init_parameters": {
            "filters": {},
            "top_k": 10,
            "document_store": {
                "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",  # noqa: E501
                "init_parameters": {
                    "azure_endpoint": {
                        "type": "env_var",
                        "env_vars": ["AZURE_AI_SEARCH_ENDPOINT"],
                        "strict": True,
                    },
                    "api_key": {"type": "env_var", "env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False},
                    "index_name": "default",
                    "embedding_dimension": 768,
                    "metadata_fields": {},
                    "vector_search_configuration": {
                        "profiles": [
                            {"name": "default-vector-config", "algorithm_configuration_name": "cosine-algorithm-config"}
                        ],
                        "algorithms": [
                            {
                                "name": "cosine-algorithm-config",
                                "kind": "hnsw",
                                "parameters": {"m": 4, "ef_construction": 400, "ef_search": 500, "metric": "cosine"},
                            }
                        ],
                    },
                    "hosts": "some fake host",
                },
            },
            "filter_policy": "replace",
        },
    }


def test_from_dict():
    data = {
        "type": "haystack_integrations.components.retrievers.azure_ai_search.hybrid_retriever.AzureAISearchHybridRetriever",  # noqa: E501
        "init_parameters": {
            "filters": {},
            "top_k": 10,
            "document_store": {
                "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",  # noqa: E501
                "init_parameters": {
                    "azure_endpoint": {
                        "type": "env_var",
                        "env_vars": ["AZURE_AI_SEARCH_ENDPOINT"],
                        "strict": True,
                    },
                    "api_key": {"type": "env_var", "env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False},
                    "index_name": "default",
                    "embedding_dimension": 768,
                    "metadata_fields": None,
                    "vector_search_configuration": DEFAULT_VECTOR_SEARCH,
                    "hosts": "some fake host",
                },
            },
            "filter_policy": "replace",
        },
    }
    retriever = AzureAISearchHybridRetriever.from_dict(data)
    assert isinstance(retriever._document_store, AzureAISearchDocumentStore)
    assert retriever._filters == {}
    assert retriever._top_k == 10
    assert retriever._filter_policy == FilterPolicy.REPLACE


def test_run():
    mock_store = Mock(spec=AzureAISearchDocumentStore)
    mock_store._hybrid_retrieval.return_value = [Document(content="Test doc", embedding=[0.1, 0.2])]
    retriever = AzureAISearchHybridRetriever(document_store=mock_store)
    res = retriever.run(query_embedding=[0.5, 0.7], query="Test query")
    mock_store._hybrid_retrieval.assert_called_once_with(
        query="Test query",
        query_embedding=[0.5, 0.7],
        filters="",
        top_k=10,
    )
    assert len(res) == 1
    assert len(res["documents"]) == 1
    assert res["documents"][0].content == "Test doc"
    assert res["documents"][0].embedding == [0.1, 0.2]


def test_run_init_params():
    mock_store = Mock(spec=AzureAISearchDocumentStore)
    mock_store._hybrid_retrieval.return_value = [Document(content="Test doc", embedding=[0.1, 0.2])]
    retriever = AzureAISearchHybridRetriever(
        document_store=mock_store,
        filters={"field": "type", "operator": "==", "value": "article"},
        top_k=11,
    )
    res = retriever.run(query_embedding=[0.5, 0.7], query="Test query")
    mock_store._hybrid_retrieval.assert_called_once_with(
        query="Test query",
        query_embedding=[0.5, 0.7],
        filters="type eq 'article'",
        top_k=11,
    )
    assert len(res) == 1
    assert len(res["documents"]) == 1
    assert res["documents"][0].content == "Test doc"
    assert res["documents"][0].embedding == [0.1, 0.2]


def test_run_time_params():
    mock_store = Mock(spec=AzureAISearchDocumentStore)
    mock_store._hybrid_retrieval.return_value = [Document(content="Test doc", embedding=[0.1, 0.2])]
    retriever = AzureAISearchHybridRetriever(
        document_store=mock_store,
        filters={"field": "type", "operator": "==", "value": "article"},
        top_k=11,
        select="name",
    )
    res = retriever.run(
        query_embedding=[0.5, 0.7],
        query="Test query",
        filters={"field": "type", "operator": "==", "value": "book"},
        top_k=9,
    )
    mock_store._hybrid_retrieval.assert_called_once_with(
        query="Test query",
        query_embedding=[0.5, 0.7],
        filters="type eq 'book'",
        top_k=9,
        select="name",
    )
    assert len(res) == 1
    assert len(res["documents"]) == 1
    assert res["documents"][0].content == "Test doc"
    assert res["documents"][0].embedding == [0.1, 0.2]


@pytest.mark.skipif(
    not os.environ.get("AZURE_AI_SEARCH_ENDPOINT", None) and not os.environ.get("AZURE_AI_SEARCH_API_KEY", None),
    reason="Missing AZURE_AI_SEARCH_ENDPOINT or AZURE_AI_SEARCH_API_KEY.",
)
@pytest.mark.integration
class TestRetriever:
    def test_run(self, document_store: AzureAISearchDocumentStore):
        docs = [Document(id="1")]
        document_store.write_documents(docs)
        retriever = AzureAISearchHybridRetriever(document_store=document_store)
        res = retriever.run(query="Test document", query_embedding=[0.1] * 768)
        assert res["documents"][0].id == docs[0].id

    def test_hybrid_retrieval(self, document_store: AzureAISearchDocumentStore):
        query_embedding = [0.1] * 768
        most_similar_embedding = [0.8] * 768
        second_best_embedding = [0.8] * 200 + [0.1] * 300 + [0.2] * 268
        another_embedding = rand(768).tolist()

        docs = [
            Document(content="This is first document", embedding=most_similar_embedding),
            Document(content="This is second document", embedding=second_best_embedding),
            Document(content="This is third document", embedding=another_embedding),
        ]

        document_store.write_documents(docs)
        retriever = AzureAISearchHybridRetriever(document_store=document_store)
        results = retriever.run(query="This is first document", query_embedding=query_embedding)
        assert results["documents"][0].content == "This is first document"

    def test_empty_query_embedding(self, document_store: AzureAISearchDocumentStore):
        query_embedding: list[float] = []
        with pytest.raises(ValueError):
            document_store._hybrid_retrieval(query="", query_embedding=query_embedding)

    def test_query_embedding_wrong_dimension(self, document_store: AzureAISearchDocumentStore):
        query_embedding = [0.1] * 4
        with pytest.raises(HttpResponseError):
            document_store._hybrid_retrieval(query="", query_embedding=query_embedding)



# SPDX-FileCopyrightText: 2023-present deepset GmbH <info@deepset.ai>
#
# SPDX-License-Identifier: Apache-2.0
from .document_store import DEFAULT_VECTOR_SEARCH, AzureAISearchDocumentStore
from .filters import _normalize_filters

__all__ = ["DEFAULT_VECTOR_SEARCH", "AzureAISearchDocumentStore", "_normalize_filters"]



# SPDX-FileCopyrightText: 2023-present deepset GmbH <info@deepset.ai>
#
# SPDX-License-Identifier: Apache-2.0

import os
import random
from datetime import datetime, timezone
from unittest.mock import patch

import pytest
from azure.search.documents.indexes.models import CustomAnalyzer, SearchField, SearchResourceEncryptionKey, SimpleField
from haystack.dataclasses.document import Document
from haystack.errors import FilterError
from haystack.testing.document_store import (
    CountDocumentsTest,
    DeleteDocumentsTest,
    FilterDocumentsTest,
    WriteDocumentsTest,
)
from haystack.utils.auth import EnvVarSecret, Secret

from haystack_integrations.document_stores.azure_ai_search import (
    DEFAULT_VECTOR_SEARCH,
    AzureAISearchDocumentStore,
)


def test_to_dict(monkeypatch):
    monkeypatch.setenv("AZURE_AI_SEARCH_API_KEY", "test-api-key")
    monkeypatch.setenv("AZURE_AI_SEARCH_ENDPOINT", "test-endpoint")
    document_store = AzureAISearchDocumentStore()
    res = document_store.to_dict()
    assert res == {
        "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",
        "init_parameters": {
            "azure_endpoint": {"env_vars": ["AZURE_AI_SEARCH_ENDPOINT"], "strict": True, "type": "env_var"},
            "api_key": {"env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False, "type": "env_var"},
            "index_name": "default",
            "embedding_dimension": 768,
            "metadata_fields": {},
            "vector_search_configuration": {
                "profiles": [
                    {"name": "default-vector-config", "algorithm_configuration_name": "cosine-algorithm-config"}
                ],
                "algorithms": [
                    {
                        "name": "cosine-algorithm-config",
                        "kind": "hnsw",
                        "parameters": {"m": 4, "ef_construction": 400, "ef_search": 500, "metric": "cosine"},
                    }
                ],
            },
        },
    }


def test_to_dict_with_params(monkeypatch):
    monkeypatch.setenv("AZURE_AI_SEARCH_API_KEY", "test-api-key")
    monkeypatch.setenv("AZURE_AI_SEARCH_ENDPOINT", "test-endpoint")
    encryption_key = SearchResourceEncryptionKey(
        key_name="my-key",
        key_version="my-version",
        vault_uri="my-uri",
    )
    analyzer = CustomAnalyzer(
        name="url-analyze",
        tokenizer_name="uax_url_email",
        token_filters=["lowercase"],  # Using token filter name directly as string
    )
    document_store = AzureAISearchDocumentStore(
        index_name="my_index",
        embedding_dimension=15,
        metadata_fields={
            "Title": SearchField(name="Title", type="Edm.String", searchable=True, filterable=True),
            "Pages": int,
        },
        encryption_key=encryption_key,
        analyzers=[analyzer],
    )

    res = document_store.to_dict()
    assert res == {
        "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",
        "init_parameters": {
            "azure_endpoint": {"env_vars": ["AZURE_AI_SEARCH_ENDPOINT"], "strict": True, "type": "env_var"},
            "api_key": {"env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False, "type": "env_var"},
            "index_name": "my_index",
            "embedding_dimension": 15,
            "metadata_fields": {
                "Title": SearchField(name="Title", type="Edm.String", searchable=True, filterable=True).as_dict(),
                "Pages": SimpleField(name="Pages", type="Edm.Int32", filterable=True).as_dict(),
            },
            "encryption_key": {
                "key_name": "my-key",
                "key_version": "my-version",
                "vault_uri": "my-uri",
            },
            "analyzers": [
                {
                    "name": "url-analyze",
                    "odata_type": "#Microsoft.Azure.Search.CustomAnalyzer",
                    "tokenizer_name": "uax_url_email",
                    "token_filters": ["lowercase"],
                }
            ],
            "vector_search_configuration": {
                "profiles": [
                    {"name": "default-vector-config", "algorithm_configuration_name": "cosine-algorithm-config"}
                ],
                "algorithms": [
                    {
                        "name": "cosine-algorithm-config",
                        "kind": "hnsw",
                        "parameters": {"m": 4, "ef_construction": 400, "ef_search": 500, "metric": "cosine"},
                    }
                ],
            },
        },
    }


def test_from_dict(monkeypatch):
    monkeypatch.setenv("AZURE_AI_SEARCH_API_KEY", "test-api-key")
    monkeypatch.setenv("AZURE_AI_SEARCH_ENDPOINT", "test-endpoint")

    data = {
        "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",
        "init_parameters": {
            "azure_endpoint": {"env_vars": ["AZURE_AI_SEARCH_ENDPOINT"], "strict": True, "type": "env_var"},
            "api_key": {"env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False, "type": "env_var"},
            "embedding_dimension": 768,
            "index_name": "default",
            "metadata_fields": None,
            "vector_search_configuration": DEFAULT_VECTOR_SEARCH,
        },
    }
    document_store = AzureAISearchDocumentStore.from_dict(data)
    assert isinstance(document_store._api_key, EnvVarSecret)
    assert isinstance(document_store._azure_endpoint, EnvVarSecret)
    assert document_store._index_name == "default"
    assert document_store._embedding_dimension == 768
    assert document_store._metadata_fields == {}
    assert document_store._vector_search_configuration == DEFAULT_VECTOR_SEARCH


def test_from_dict_with_params(monkeypatch):
    monkeypatch.setenv("AZURE_AI_SEARCH_API_KEY", "test-api-key")
    monkeypatch.setenv("AZURE_AI_SEARCH_ENDPOINT", "test-endpoint")
    encryption_key = SearchResourceEncryptionKey(
        key_name="my-key",
        key_version="my-version",
        vault_uri="my-uri",
    )

    data = {
        "type": "haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore",
        "init_parameters": {
            "azure_endpoint": {"env_vars": ["AZURE_AI_SEARCH_ENDPOINT"], "strict": True, "type": "env_var"},
            "api_key": {"env_vars": ["AZURE_AI_SEARCH_API_KEY"], "strict": False, "type": "env_var"},
            "index_name": "my_index",
            "embedding_dimension": 15,
            "metadata_fields": {
                "Title": SearchField(name="Title", type="Edm.String", filterable=True).as_dict(),
                "Pages": SimpleField(name="Pages", type="Edm.Int32", filterable=True).as_dict(),
            },
            "encryption_key": {
                "key_name": "my-key",
                "key_version": "my-version",
                "vault_uri": "my-uri",
            },
            "analyzers": [
                {
                    "name": "url-analyze",
                    "odata_type": "#Microsoft.Azure.Search.CustomAnalyzer",
                    "tokenizer_name": "uax_url_email",
                    "token_filters": ["lowercase"],
                }
            ],
            "vector_search_configuration": {
                "profiles": [
                    {"name": "default-vector-config", "algorithm_configuration_name": "cosine-algorithm-config"}
                ],
                "algorithms": [
                    {
                        "name": "cosine-algorithm-config",
                        "kind": "hnsw",
                        "parameters": {"m": 4, "ef_construction": 400, "ef_search": 500, "metric": "cosine"},
                    }
                ],
            },
        },
    }
    document_store = AzureAISearchDocumentStore.from_dict(data)
    assert isinstance(document_store._api_key, EnvVarSecret)
    assert isinstance(document_store._azure_endpoint, EnvVarSecret)
    assert document_store._index_name == "my_index"
    assert document_store._embedding_dimension == 15
    assert document_store._metadata_fields == {
        "Title": SearchField(name="Title", type="Edm.String", filterable=True),
        "Pages": SimpleField(name="Pages", type="Edm.Int32", filterable=True),
    }
    assert document_store._index_creation_kwargs["encryption_key"] == encryption_key
    assert document_store._index_creation_kwargs["analyzers"][0].name == "url-analyze"
    assert document_store._index_creation_kwargs["analyzers"][0].token_filters == ["lowercase"]
    assert "CustomAnalyzer" in document_store._index_creation_kwargs["analyzers"][0].odata_type
    assert document_store._vector_search_configuration.as_dict() == DEFAULT_VECTOR_SEARCH.as_dict()


@patch("haystack_integrations.document_stores.azure_ai_search.document_store.AzureAISearchDocumentStore")
def test_init_is_lazy(_mock_azure_search_client):
    AzureAISearchDocumentStore(azure_endpoint=Secret.from_token("test_endpoint"))
    _mock_azure_search_client.assert_not_called()


def test_init():
    document_store = AzureAISearchDocumentStore(
        api_key=Secret.from_token("fake-api-key"),
        azure_endpoint=Secret.from_token("fake_endpoint"),
        index_name="my_index",
        embedding_dimension=15,
        metadata_fields={"Title": str, "Pages": int},
    )

    assert document_store._index_name == "my_index"
    assert document_store._embedding_dimension == 15
    assert document_store._metadata_fields == {
        "Title": SimpleField(name="Title", type="Edm.String", filterable=True),
        "Pages": SimpleField(name="Pages", type="Edm.Int32", filterable=True),
    }
    assert document_store._vector_search_configuration == DEFAULT_VECTOR_SEARCH


def _assert_documents_are_equal(received: list[Document], expected: list[Document]):
    """
    Assert that two lists of Documents are equal.

    This is used in every test, if a Document Store implementation has a different behaviour
    it should override this method. This can happen for example when the Document Store sets
    a score to returned Documents. Since we can't know what the score will be, we can't compare
    the Documents reliably.
    """
    sorted_received = sorted(received, key=lambda doc: doc.id)
    sorted_expected = sorted(expected, key=lambda doc: doc.id)
    assert len(sorted_received) == len(sorted_expected)

    for received_doc, expected_doc in zip(sorted_received, sorted_expected, strict=True):
        # Compare all attributes except score
        assert received_doc.id == expected_doc.id
        assert received_doc.content == expected_doc.content
        assert received_doc.embedding == expected_doc.embedding
        assert received_doc.meta == expected_doc.meta


@pytest.mark.integration
@pytest.mark.skipif(
    not os.environ.get("AZURE_AI_SEARCH_ENDPOINT", None) and not os.environ.get("AZURE_AI_SEARCH_API_KEY", None),
    reason="Missing AZURE_AI_SEARCH_ENDPOINT or AZURE_AI_SEARCH_API_KEY.",
)
class TestDocumentStore(CountDocumentsTest, WriteDocumentsTest, DeleteDocumentsTest):
    def assert_documents_are_equal(self, received: list[Document], expected: list[Document]):
        _assert_documents_are_equal(received, expected)

    def test_write_documents(self, document_store: AzureAISearchDocumentStore):
        docs = [Document(id="1")]
        assert document_store.write_documents(docs) == 1

    # Parametrize the test with metadata fields
    @pytest.mark.parametrize(
        "document_store",
        [
            {"metadata_fields": {"author": str, "publication_year": int, "rating": float}},
        ],
        indirect=True,
    )
    def test_write_documents_with_meta(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(
                id="1",
                meta={"author": "Tom", "publication_year": 2021, "rating": 4.5},
                content="This is a test document.",
            )
        ]
        document_store.write_documents(docs)
        doc = document_store.get_documents_by_id(["1"])
        assert doc[0] == docs[0]

    @pytest.mark.skip(reason="Azure AI search index overwrites duplicate documents by default")
    def test_write_documents_duplicate_fail(self, document_store: AzureAISearchDocumentStore): ...

    @pytest.mark.skip(reason="Azure AI search index overwrites duplicate documents by default")
    def test_write_documents_duplicate_skip(self, document_store: AzureAISearchDocumentStore): ...

    def test_delete_all_documents(self, document_store: AzureAISearchDocumentStore):
        docs = [Document(content="first doc"), Document(content="second doc")]
        document_store.write_documents(docs)
        assert document_store.count_documents() == 2
        document_store.delete_all_documents()
        assert document_store.count_documents() == 0

    def test_delete_all_documents_empty_index(self, document_store: AzureAISearchDocumentStore):
        assert document_store.count_documents() == 0
        document_store.delete_all_documents()
        assert document_store.count_documents() == 0

    @pytest.mark.parametrize(
        "document_store",
        [{"metadata_fields": {"category": str}}],
        indirect=True,
    )
    def test_delete_by_filter(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(content="Doc 1", meta={"category": "A"}),
            Document(content="Doc 2", meta={"category": "B"}),
            Document(content="Doc 3", meta={"category": "A"}),
        ]
        document_store.write_documents(docs)
        assert document_store.count_documents() == 3

        # Delete documents with category="A"
        deleted_count = document_store.delete_by_filter(
            filters={"field": "meta.category", "operator": "==", "value": "A"}
        )
        assert deleted_count == 2
        assert document_store.count_documents() == 1

        # Verify only category B remains
        remaining_docs = document_store.filter_documents()
        assert len(remaining_docs) == 1
        assert remaining_docs[0].meta["category"] == "B"

    @pytest.mark.parametrize(
        "document_store",
        [{"metadata_fields": {"category": str}}],
        indirect=True,
    )
    def test_delete_by_filter_no_matches(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(content="Doc 1", meta={"category": "A"}),
            Document(content="Doc 2", meta={"category": "B"}),
        ]
        document_store.write_documents(docs)
        assert document_store.count_documents() == 2

        # Try to delete documents with category="C" (no matches)
        deleted_count = document_store.delete_by_filter(
            filters={"field": "meta.category", "operator": "==", "value": "C"}
        )
        assert deleted_count == 0
        assert document_store.count_documents() == 2

    @pytest.mark.parametrize(
        "document_store",
        [{"metadata_fields": {"category": str, "status": str}}],
        indirect=True,
    )
    def test_update_by_filter(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(content="Doc 1", meta={"category": "A", "status": "draft"}),
            Document(content="Doc 2", meta={"category": "B", "status": "draft"}),
            Document(content="Doc 3", meta={"category": "A", "status": "draft"}),
        ]
        document_store.write_documents(docs)
        assert document_store.count_documents() == 3

        # Update status for category="A" documents
        updated_count = document_store.update_by_filter(
            filters={"field": "meta.category", "operator": "==", "value": "A"},
            fields={"status": "published"},
        )
        assert updated_count == 2

        # Verify the updates
        published_docs = document_store.filter_documents(
            filters={"field": "meta.status", "operator": "==", "value": "published"}
        )
        assert len(published_docs) == 2
        for doc in published_docs:
            assert doc.meta["category"] == "A"
            assert doc.meta["status"] == "published"

        # Verify category B still has draft status
        draft_docs = document_store.filter_documents(
            filters={"field": "meta.status", "operator": "==", "value": "draft"}
        )
        assert len(draft_docs) == 1
        assert draft_docs[0].meta["category"] == "B"

    @pytest.mark.parametrize(
        "document_store",
        [{"metadata_fields": {"category": str, "status": str}}],
        indirect=True,
    )
    def test_update_by_filter_no_matches(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(content="Doc 1", meta={"category": "A", "status": "draft"}),
            Document(content="Doc 2", meta={"category": "B", "status": "draft"}),
        ]
        document_store.write_documents(docs)
        assert document_store.count_documents() == 2

        # Try to update documents with category="C" (no matches)
        updated_count = document_store.update_by_filter(
            filters={"field": "meta.category", "operator": "==", "value": "C"},
            fields={"status": "published"},
        )
        assert updated_count == 0

    @pytest.mark.parametrize(
        "document_store",
        [{"metadata_fields": {"category": str, "status": str}}],
        indirect=True,
    )
    def test_update_by_filter_invalid_field(self, document_store: AzureAISearchDocumentStore):
        docs = [
            Document(content="Doc 1", meta={"category": "A", "status": "draft"}),
        ]
        document_store.write_documents(docs)

        # Try to update a field that doesn't exist in the schema
        with pytest.raises(ValueError) as exc_info:
            document_store.update_by_filter(
                filters={"field": "meta.category", "operator": "==", "value": "A"},
                fields={"nonexistent_field": "value"},
            )
        assert "nonexistent_field" in str(exc_info.value)
        assert "not defined in index schema" in str(exc_info.value)


def _random_embeddings(n):
    return [round(random.random(), 7) for _ in range(n)]  # nosec: S311


TEST_EMBEDDING_1 = _random_embeddings(768)
TEST_EMBEDDING_2 = _random_embeddings(768)


@pytest.mark.integration
@pytest.mark.skipif(
    not os.environ.get("AZURE_AI_SEARCH_ENDPOINT", None) and not os.environ.get("AZURE_AI_SEARCH_API_KEY", None),
    reason="Missing AZURE_AI_SEARCH_ENDPOINT or AZURE_AI_SEARCH_API_KEY.",
)
@pytest.mark.parametrize(
    "document_store",
    [
        {"metadata_fields": {"name": str, "page": str, "chapter": str, "number": int, "date": datetime}},
    ],
    indirect=True,
)
class TestFilters(FilterDocumentsTest):
    # Overriding to change "date" to compatible ISO 8601 format
    @pytest.fixture
    def filterable_docs(self) -> list[Document]:
        """Fixture that returns a list of Documents that can be used to test filtering."""
        documents = []
        for i in range(3):
            documents.append(
                Document(
                    content=f"A Foo Document {i}",
                    meta={
                        "name": f"name_{i}",
                        "page": "100",
                        "chapter": "intro",
                        "number": 2,
                        "date": "1969-07-21T20:17:40Z",
                    },
                    embedding=_random_embeddings(768),
                )
            )
            documents.append(
                Document(
                    content=f"A Bar Document {i}",
                    meta={
                        "name": f"name_{i}",
                        "page": "123",
                        "chapter": "abstract",
                        "number": -2,
                        "date": "1972-12-11T19:54:58Z",
                    },
                    embedding=_random_embeddings(768),
                )
            )
            documents.append(
                Document(
                    content=f"A Foobar Document {i}",
                    meta={
                        "name": f"name_{i}",
                        "page": "90",
                        "chapter": "conclusion",
                        "number": -10,
                        "date": "1989-11-09T17:53:00Z",
                    },
                    embedding=_random_embeddings(768),
                )
            )

            documents.append(
                Document(content=f"Doc {i} with zeros emb", meta={"name": "zeros_doc"}, embedding=TEST_EMBEDDING_1)
            )
            documents.append(
                Document(content=f"Doc {i} with ones emb", meta={"name": "ones_doc"}, embedding=TEST_EMBEDDING_2)
            )
        return documents

    # Overriding to compare the documents with the same order
    def assert_documents_are_equal(self, received: list[Document], expected: list[Document]):
        _assert_documents_are_equal(received, expected)

    # Azure search index supports UTC datetime in ISO 8601 format
    def test_comparison_greater_than_with_iso_date(self, document_store, filterable_docs):
        """Test filter_documents() with > comparator and datetime"""
        document_store.write_documents(filterable_docs)
        result = document_store.filter_documents(
            {"field": "meta.date", "operator": ">", "value": "1972-12-11T19:54:58Z"}
        )
        self.assert_documents_are_equal(
            result,
            [
                d
                for d in filterable_docs
                if d.meta.get("date") is not None
                and datetime.strptime(d.meta["date"], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
                > datetime.strptime("1972-12-11T19:54:58Z", "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
            ],
        )

    def test_comparison_greater_than_equal_with_iso_date(self, document_store, filterable_docs):
        """Test filter_documents() with >= comparator and datetime"""
        document_store.write_documents(filterable_docs)
        result = document_store.filter_documents(
            {"field": "meta.date", "operator": ">=", "value": "1969-07-21T20:17:40Z"}
        )
        self.assert_documents_are_equal(
            result,
            [
                d
                for d in filterable_docs
                if d.meta.get("date") is not None
                and datetime.strptime(d.meta["date"], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
                >= datetime.strptime("1969-07-21T20:17:40Z", "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
            ],
        )

    def test_comparison_less_than_with_iso_date(self, document_store, filterable_docs):
        """Test filter_documents() with < comparator and datetime"""
        document_store.write_documents(filterable_docs)
        result = document_store.filter_documents(
            {"field": "meta.date", "operator": "<", "value": "1969-07-21T20:17:40Z"}
        )
        self.assert_documents_are_equal(
            result,
            [
                d
                for d in filterable_docs
                if d.meta.get("date") is not None
                and datetime.strptime(d.meta["date"], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
                < datetime.strptime("1969-07-21T20:17:40Z", "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
            ],
        )

    def test_comparison_less_than_equal_with_iso_date(self, document_store, filterable_docs):
        """Test filter_documents() with <= comparator and datetime"""
        document_store.write_documents(filterable_docs)
        result = document_store.filter_documents(
            {"field": "meta.date", "operator": "<=", "value": "1969-07-21T20:17:40Z"}
        )
        self.assert_documents_are_equal(
            result,
            [
                d
                for d in filterable_docs
                if d.meta.get("date") is not None
                and datetime.strptime(d.meta["date"], "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
                <= datetime.strptime("1969-07-21T20:17:40Z", "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=timezone.utc)
            ],
        )

    # Override as comparison operators with None/null raise errors
    def test_comparison_greater_than_with_none(self, document_store, filterable_docs):
        """Test filter_documents() with > comparator and None"""
        document_store.write_documents(filterable_docs)
        with pytest.raises(FilterError):
            document_store.filter_documents(filters={"field": "meta.number", "operator": ">", "value": None})

    def test_comparison_greater_than_equal_with_none(self, document_store, filterable_docs):
        """Test filter_documents() with >= comparator and None"""
        document_store.write_documents(filterable_docs)
        with pytest.raises(FilterError):
            document_store.filter_documents(filters={"field": "meta.number", "operator": ">=", "value": None})

    def test_comparison_less_than_with_none(self, document_store, filterable_docs):
        """Test filter_documents() with < comparator and None"""
        document_store.write_documents(filterable_docs)
        with pytest.raises(FilterError):
            document_store.filter_documents(filters={"field": "meta.number", "operator": "<", "value": None})

    def test_comparison_less_than_equal_with_none(self, document_store, filterable_docs):
        """Test filter_documents() with <= comparator and None"""
        document_store.write_documents(filterable_docs)
        with pytest.raises(FilterError):
            document_store.filter_documents(filters={"field": "meta.number", "operator": "<=", "value": None})

    # Override as Azure AI Search supports 'in' operator only for strings
    def test_comparison_in(self, document_store, filterable_docs):
        """Test filter_documents() with 'in' comparator"""
        document_store.write_documents(filterable_docs)
        result = document_store.filter_documents({"field": "meta.page", "operator": "in", "value": ["100", "123"]})
        assert len(result)
        expected = [d for d in filterable_docs if d.meta.get("page") is not None and d.meta["page"] in ["100", "123"]]
        self.assert_documents_are_equal(result, expected)

    @pytest.mark.skip(reason="Azure AI search index does not support not in operator")
    def test_comparison_not_in(self, document_store, filterable_docs): ...

    @pytest.mark.skip(reason="Azure AI search index does not support not in operator")
    def test_comparison_not_in_with_with_non_list(self, document_store, filterable_docs): ...

    @pytest.mark.skip(reason="Azure AI search index does not support not in operator")
    def test_comparison_not_in_with_with_non_list_iterable(self, document_store, filterable_docs): ...

    def test_missing_condition_operator_key(self, document_store, filterable_docs):
        """Test filter_documents() with missing operator key"""
        document_store.write_documents(filterable_docs)
        with pytest.raises(FilterError):
            document_store.filter_documents(
                filters={"conditions": [{"field": "meta.name", "operator": "eq", "value": "test"}]}
            )

    def test_nested_logical_filters(self, document_store, filterable_docs):
        document_store.write_documents(filterable_docs)
        filters = {
            "operator": "OR",
            "conditions": [
                {"field": "meta.name", "operator": "==", "value": "name_0"},
                {
                    "operator": "AND",
                    "conditions": [
                        {"field": "meta.number", "operator": "!=", "value": 0},
                        {"field": "meta.page", "operator": "==", "value": "123"},
                    ],
                },
                {
                    "operator": "AND",
                    "conditions": [
                        {"field": "meta.chapter", "operator": "==", "value": "conclusion"},
                        {"field": "meta.page", "operator": "==", "value": "90"},
                    ],
                },
            ],
        }
        result = document_store.filter_documents(filters=filters)
        self.assert_documents_are_equal(
            result,
            [
                doc
                for doc in filterable_docs
                if (
                    # Ensure all required fields are present in doc.meta
                    ("name" in doc.meta and doc.meta.get("name") == "name_0")
                    or (
                        all(key in doc.meta for key in ["number", "page"])
                        and doc.meta.get("number") != 0
                        and doc.meta.get("page") == "123"
                    )
                    or (
                        all(key in doc.meta for key in ["page", "chapter"])
                        and doc.meta.get("chapter") == "conclusion"
                        and doc.meta.get("page") == "90"
                    )
                )
            ],
        )



from haystack import Document, Pipeline
from haystack.components.embedders import SentenceTransformersDocumentEmbedder, SentenceTransformersTextEmbedder
from haystack.components.writers import DocumentWriter

from haystack_integrations.components.retrievers.azure_ai_search import AzureAISearchEmbeddingRetriever
from haystack_integrations.document_stores.azure_ai_search import AzureAISearchDocumentStore

"""
This example demonstrates how to use the AzureAISearchEmbeddingRetriever to retrieve documents
using embeddings based on a query. To run this example, you'll need an Azure Search service endpoint
and API key, which can either be
set as environment variables (AZURE_AI_SEARCH_ENDPOINT and AZURE_AI_SEARCH_API_KEY) or
provided directly to AzureAISearchDocumentStore(as params "api_key", "azure_endpoint").
Otherwise you can use DefaultAzureCredential to authenticate with Azure services.
See more details at https://learn.microsoft.com/en-us/azure/search/keyless-connections?tabs=python%2Cazure-cli
"""

document_store = AzureAISearchDocumentStore(index_name="retrieval-example")

model = "sentence-transformers/all-mpnet-base-v2"

documents = [
    Document(content="There are over 7,000 languages spoken around the world today."),
    Document(
        content="""Elephants have been observed to behave in a way that indicates a
         high level of self-awareness, such as recognizing themselves in mirrors."""
    ),
    Document(
        content="""In certain parts of the world, like the Maldives, Puerto Rico, and
          San Diego, you can witness the phenomenon of bioluminescent waves."""
    ),
]

document_embedder = SentenceTransformersDocumentEmbedder(model=model)
document_embedder.warm_up()

# Indexing Pipeline
indexing_pipeline = Pipeline()
indexing_pipeline.add_component(instance=document_embedder, name="doc_embedder")
indexing_pipeline.add_component(instance=DocumentWriter(document_store=document_store), name="doc_writer")
indexing_pipeline.connect("doc_embedder", "doc_writer")

indexing_pipeline.run({"doc_embedder": {"documents": documents}})

# Query Pipeline
query_pipeline = Pipeline()
query_pipeline.add_component("text_embedder", SentenceTransformersTextEmbedder(model=model))
query_pipeline.add_component("retriever", AzureAISearchEmbeddingRetriever(document_store=document_store))
query_pipeline.connect("text_embedder.embedding", "retriever.query_embedding")

query = "How many languages are there?"

result = query_pipeline.run({"text_embedder": {"text": query}})

print(result["retriever"]["documents"][0])



from typing import Any

from dateutil import parser

from .errors import AzureAISearchDocumentStoreFilterError

LOGICAL_OPERATORS = {"AND": "and", "OR": "or", "NOT": "not"}


def _normalize_filters(filters: dict[str, Any]) -> str:
    """
    Converts Haystack filters in Azure AI Search compatible filters.
    """
    if not isinstance(filters, dict):
        msg = """Filters must be a dictionary.
        See https://docs.haystack.deepset.ai/docs/metadata-filtering for details on filters syntax."""
        raise AzureAISearchDocumentStoreFilterError(msg)

    if "field" in filters:
        return _parse_comparison_condition(filters)
    return _parse_logical_condition(filters)


def _parse_logical_condition(condition: dict[str, Any]) -> str:
    missing_keys = [key for key in ("operator", "conditions") if key not in condition]
    if missing_keys:
        msg = f"""Missing key(s) {missing_keys} in {condition}.
        See https://docs.haystack.deepset.ai/docs/metadata-filtering for details on filters syntax."""
        raise AzureAISearchDocumentStoreFilterError(msg)

    operator = condition["operator"]
    if operator not in LOGICAL_OPERATORS:
        msg = f"Unknown operator {operator}"
        raise AzureAISearchDocumentStoreFilterError(msg)
    conditions = []
    for c in condition["conditions"]:
        # Recursively parse if the condition itself is a logical condition
        if isinstance(c, dict) and "operator" in c and c["operator"] in LOGICAL_OPERATORS:
            conditions.append(_parse_logical_condition(c))
        else:
            # Otherwise, parse it as a comparison condition
            conditions.append(_parse_comparison_condition(c))

    # Format the result based on the operator
    if operator == "NOT":
        return f"not ({' and '.join([f'({c})' for c in conditions])})"
    else:
        return f" {LOGICAL_OPERATORS[operator]} ".join([f"({c})" for c in conditions])


def _parse_comparison_condition(condition: dict[str, Any]) -> str:
    missing_keys = [key for key in ("field", "operator", "value") if key not in condition]
    if missing_keys:
        msg = f"""Missing key(s) {missing_keys} in {condition}.
        See https://docs.haystack.deepset.ai/docs/metadata-filtering for details on filters syntax."""
        raise AzureAISearchDocumentStoreFilterError(msg)

    # Remove the "meta." prefix from the field name if present
    field = condition["field"][5:] if condition["field"].startswith("meta.") else condition["field"]
    operator = condition["operator"]
    value = "null" if condition["value"] is None else condition["value"]

    if operator not in COMPARISON_OPERATORS:
        msg = f"Unknown operator {operator}. Valid operators are: {list(COMPARISON_OPERATORS.keys())}"
        raise AzureAISearchDocumentStoreFilterError(msg)

    return COMPARISON_OPERATORS[operator](field, value)


def _eq(field: str, value: Any) -> str:
    return f"{field} eq '{value}'" if isinstance(value, str) and value != "null" else f"{field} eq {value}"


def _ne(field: str, value: Any) -> str:
    return f"not ({field} eq '{value}')" if isinstance(value, str) and value != "null" else f"not ({field} eq {value})"


def _in(field: str, value: Any) -> str:
    if not isinstance(value, list) or any(not isinstance(v, str) for v in value):
        msg = "Azure AI Search only supports a list of strings for 'in' comparators"
        raise AzureAISearchDocumentStoreFilterError(msg)
    values = ",".join(map(str, value))
    return f"search.in({field},'{values}',',')"


def _comparison_operator(field: str, value: Any, operator: str) -> str:
    _validate_type(value, operator)
    return f"{field} {operator} {value}"


def _validate_type(value: Any, operator: str) -> None:
    """Validates that the value is either an integer, float, or ISO 8601 string."""
    msg = f"Invalid value type for '{operator}' comparator. Supported types are: int, float, or ISO 8601 string."

    if isinstance(value, str):
        try:
            parser.isoparse(value)
        except ValueError as e:
            raise AzureAISearchDocumentStoreFilterError(msg) from e
    elif not isinstance(value, (int, float)):
        raise AzureAISearchDocumentStoreFilterError(msg)


COMPARISON_OPERATORS = {
    "==": _eq,
    "!=": _ne,
    "in": _in,
    ">": lambda f, v: _comparison_operator(f, v, "gt"),
    ">=": lambda f, v: _comparison_operator(f, v, "ge"),
    "<": lambda f, v: _comparison_operator(f, v, "lt"),
    "<=": lambda f, v: _comparison_operator(f, v, "le"),
}



from typing import Any

from haystack import Document, component, default_from_dict, default_to_dict, logging
from haystack.document_stores.types import FilterPolicy
from haystack.document_stores.types.filter_policy import apply_filter_policy

from haystack_integrations.document_stores.azure_ai_search import AzureAISearchDocumentStore, _normalize_filters

logger = logging.getLogger(__name__)


@component
class AzureAISearchEmbeddingRetriever:
    """
    Retrieves documents from the AzureAISearchDocumentStore using a vector similarity metric.
    Must be connected to the AzureAISearchDocumentStore to run.

    """

    def __init__(
        self,
        *,
        document_store: AzureAISearchDocumentStore,
        filters: dict[str, Any] | None = None,
        top_k: int = 10,
        filter_policy: str | FilterPolicy = FilterPolicy.REPLACE,
        **kwargs: Any,
    ):
        """
        Create the AzureAISearchEmbeddingRetriever component.

        :param document_store: An instance of AzureAISearchDocumentStore to use with the Retriever.
        :param filters: Filters applied when fetching documents from the Document Store.
        :param top_k: Maximum number of documents to return.
        :param filter_policy: Policy to determine how filters are applied.
        :param kwargs: Additional keyword arguments to pass to the Azure AI's search endpoint.
            Some of the supported parameters:
                - `query_type`: A string indicating the type of query to perform. Possible values are
                'simple','full' and 'semantic'.
                - `semantic_configuration_name`: The name of semantic configuration to be used when
                processing semantic queries.
            For more information on parameters, see the
            [official Azure AI Search documentation](https://learn.microsoft.com/en-us/azure/search/).

        """
        self._filters = filters or {}
        self._top_k = top_k
        self._document_store = document_store
        self._filter_policy = (
            filter_policy if isinstance(filter_policy, FilterPolicy) else FilterPolicy.from_str(filter_policy)
        )
        self._kwargs = kwargs

        if not isinstance(document_store, AzureAISearchDocumentStore):
            message = "document_store must be an instance of AzureAISearchDocumentStore"
            raise Exception(message)

    def to_dict(self) -> dict[str, Any]:
        """
        Serializes the component to a dictionary.

        :returns:
            Dictionary with serialized data.
        """
        return default_to_dict(
            self,
            filters=self._filters,
            top_k=self._top_k,
            document_store=self._document_store.to_dict(),
            filter_policy=self._filter_policy.value,
            **self._kwargs,
        )

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "AzureAISearchEmbeddingRetriever":
        """
        Deserializes the component from a dictionary.

        :param data:
            Dictionary to deserialize from.

        :returns:
            Deserialized component.
        """
        data["init_parameters"]["document_store"] = AzureAISearchDocumentStore.from_dict(
            data["init_parameters"]["document_store"]
        )

        # Pipelines serialized with old versions of the component might not
        # have the filter_policy field.
        if "filter_policy" in data["init_parameters"]:
            data["init_parameters"]["filter_policy"] = FilterPolicy.from_str(data["init_parameters"]["filter_policy"])
        return default_from_dict(cls, data)

    @component.output_types(documents=list[Document])
    def run(
        self, query_embedding: list[float], filters: dict[str, Any] | None = None, top_k: int | None = None
    ) -> dict[str, list[Document]]:
        """Retrieve documents from the AzureAISearchDocumentStore.

        :param query_embedding: A list of floats representing the query embedding.
        :param filters: Filters applied to the retrieved Documents. The way runtime filters are applied depends on
                the `filter_policy` chosen at retriever initialization. See `__init__` method docstring for more
                details.
        :param top_k: The maximum number of documents to retrieve.
        :returns: Dictionary with the following keys:
                - `documents`: A list of documents retrieved from the AzureAISearchDocumentStore.
        """

        top_k = top_k or self._top_k
        filters = filters or self._filters

        normalized_filters = ""
        applied_filters = apply_filter_policy(self._filter_policy, self._filters, filters)
        if applied_filters:
            normalized_filters = _normalize_filters(applied_filters)

        try:
            docs = self._document_store._embedding_retrieval(
                query_embedding=query_embedding, filters=normalized_filters, top_k=top_k, **self._kwargs
            )
        except Exception as e:
            msg = (
                "An error occurred during the embedding retrieval process from the AzureAISearchDocumentStore. "
                "Ensure that the query embedding is valid and the document store is correctly configured."
            )
            raise RuntimeError(msg) from e

        return {"documents": docs}
